/*
 * Copyright (C) 2013 Jörn Magens
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This Program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file LICENSE.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * https://www.gnu.org/licenses/lgpl-2.1.txt
 *
 * Author:
 * 	Jörn Magens <shuerhaaken@googlemail.com>
 * 
 */





#include "taginfo.h"
#include "taginfo_internal.h"


#include <asfattribute.h>
#include <popularimeterframe.h>

using namespace TagInfo;



AiffInfo::AiffInfo(const String &filename) : Info(filename) {
    taglib_tag = NULL;
    if(!file_name.isEmpty() && !create_file_ref())
        printf("Error creating file ref! %s\n", filename.toCString(false));
    if(taglib_file) {
        taglib_tag = ((TagLib::RIFF::AIFF::File *) taglib_file)->tag();
        if(!taglib_tag)
            valid = false;
    }
    else {
        printf("Cant get tag from '%s'\n", file_name.toCString(false));
        taglib_tag = NULL;
        valid = false;
    }
}


AiffInfo::~AiffInfo() {
}

bool AiffInfo::create_file_ref() {
    if(file_name.isEmpty())
        return false;
    taglib_file = 
        new TagLib::RIFF::AIFF::File(file_name.toCString(false), true, TagLib::AudioProperties::Fast);
    if(taglib_file) {
        return true;
    }
    else {
        printf("TagLib::File creation failed for '%s'\n", file_name.toCString(false));
        return false;
    }
}


bool AiffInfo::read(void) {
    if(Info::read()) {
            // If its a ID3v2 Tag try to load the labels
        if(taglib_tag && !taglib_tag->isEmpty()) {
            read_virtual_tags((TagLib::Tag *)taglib_tag);
        }
    }
    else {
          cout  << "Error: Could not read tags from file '" << file_name.toCString(true) << "'" <<  endl;
        return false; 
    }
    return true;
}


bool AiffInfo::write(void) {
    if(taglib_tag) {
        if(changedflag) {
            write_virtual_tags((TagLib::Tag *)taglib_tag);
        }
    }
    return Info::write();
}


bool AiffInfo::can_handle_images(void) const {
    return false; //TODO
}

bool AiffInfo::get_image(char*& data, int &data_length, ImageType &image_type) const {
    return false;
}

bool AiffInfo::set_image(const char* data, int data_length, ImageType image_type) {
    return false;
}


bool AiffInfo::can_handle_lyrics(void) const {
    return false;
}


String AiffInfo::get_lyrics(void) const {
    return "";
}


bool AiffInfo::set_lyrics(const String &lyrics) {
    return false;
}






